<template>
	<HMContainer type="card" v-if="supervisorMgrShow">
		<template #header>
			<el-form ref="queryFormRef" :inline="true" :model="queryParams">
				<el-form-item label="标题" prop="title">
					<el-input v-model="queryParams.title" style="width: 226.4px" placeholder="标题" clearable />
				</el-form-item>

				<el-form-item>
					<el-button type="primary" icon="ele-Search" @click="handleQueryTable"> 查询 </el-button>
					<el-button icon="ele-Refresh" @click="resetQuery">重置 </el-button>
					<el-button icon="ele-Plus" @click="openOptDlg()"> 添加 </el-button>
				</el-form-item>
			</el-form>
		</template>
		<template #main>
			<!-- <div class="h-screen">
				<p>vue-json-editor</p>
				<JsonEditorVue v-model="jsonMsg"></JsonEditorVue>
			</div> -->
			<div class="h-full" ref="chatDragContainerRef">
				<el-table
					v-loading="tableLoading"
					ref="draggableTableRef"
					class="h100"
					border
					:row-class-name="isDragStatus ? 'cursor-move' : 'cursor-pointer'"
					:data="displayTableData"
					highlight-current-row
				>
					<!-- <el-table-column type="expand" fixed="left">
					<template #default="scope">
						<div class="px-4">
							<h3 class="mb-2">页面参数</h3>
							<JsonEditor currentMode="view" :modelValue="scope.row.args" />
						</div>
					</template>
				</el-table-column> -->
					<el-table-column prop="id" label="id" width="130" fixed="left" show-overflow-tooltip> </el-table-column>

					<el-table-column prop="title" label="标题" width="300" fixed="left" show-overflow-tooltip> </el-table-column>
					<el-table-column prop="prompt" label="提示词" show-overflow-tooltip> </el-table-column>
					<el-table-column prop="published" label="状态" width="120" show-overflow-tooltip>
						<template #default="scope">
							<el-select
								:modelValue="displayTableData[scope.$index].published"
								@change="(value) => publishStatusChange(value, scope.row.id, scope.$index)"
							>
								<el-option
									v-for="item in Object.keys(supervisorPublishedMap)"
									:key="item"
									:value="item"
									:label="supervisorPublishedMap[item]"
								></el-option>
							</el-select>
						</template>
					</el-table-column>
					<el-table-column label="创建人" prop="creator" width="100" show-overflow-tooltip> </el-table-column>
					<el-table-column label="创建时间" prop="create_time" width="105" show-overflow-tooltip> </el-table-column>
					<el-table-column label="更新时间" prop="update_time" width="105" show-overflow-tooltip> </el-table-column>
					<el-table-column label="备注" prop="note" show-overflow-tooltip> </el-table-column>
					<el-table-column label="操作" width="150" fixed="right" show-overflow-tooltip>
						<template #default="scope">
							<div class="space-x-2.5 items-center flex">
								<el-tooltip effect="dark" content="SQL编辑" placement="top">
									<i class="ywifont ywicon-sql !text-[14px] text-blue-400 cursor-pointer" @click="editSqlClick(scope.row)"></i>
								</el-tooltip>
								<el-tooltip effect="dark" content="AMIS低代码编辑" placement="top">
									<i class="ywifont ywicon-didaima !text-[21px] text-blue-400 cursor-pointer" @click="gotoAmisPage(scope.row)"></i>
								</el-tooltip>
								<el-tooltip effect="dark" content="对话测试" placement="top">
									<i class="ywifont ywicon-ceshi !text-[20px] text-blue-400 cursor-pointer" @click="openChatTest(scope.row)"></i>
								</el-tooltip>
								<el-tooltip effect="dark" content="编辑" placement="top">
									<i class="ywifont ywicon-bianji !text-[15px] text-blue-400 cursor-pointer" @click="openOptDlg(scope.row)"></i>
								</el-tooltip>
								<el-tooltip effect="dark" content="删除" placement="top">
									<i
										class="ywifont ywicon-shanchu !text-[17px] text-red-400 cursor-pointer"
										@click="
											() => {
												deleteCurrentRow(scope.row, '页面', supervisorAdminApi.deleteSupervisor, () => {
													const foundIndex = tableData.findIndex((item) => item === scope.row);
													foundIndex > -1 && tableData.splice(foundIndex, 1);
												});
											}
										"
									></i>
								</el-tooltip>
							</div>
						</template>
					</el-table-column>
				</el-table>
				<div
					v-if="chatTestIsShow"
					ref="draggableChatRef"
					:style="style"
					class="fixed z-50 w-[700px] h-[800px] flex flex-col bg-[rgb(239,244,253)] right-0 bottom-0 rounded-lg"
				>
					<div ref="chatDragHandlerRef" class="flex-0">
						<div class="flex items-center justify-between py-2 px-4">
							<div class="font-bold cursor-move">
								WI水务智能管家——【{{ chatTestMapRow?.title }}】测试
								<!-- <img src="/static/images/logo/logo-mini.svg" width="10" height="10" /> -->
							</div>
							<i class="ywifont ywicon-guanbi font-[10px] font-bold cursor-pointer" @click="closeChatTest"></i>
						</div>
					</div>
					<Chat ref="chatRef" class="flex-auto px-2" :questionApi="questionAi"> </Chat>
				</div>
			</div>
		</template>

		<OptDlg v-model="optDlgIsShow" :item="optDlgMapRow" @insert="insertOpt" @update="updateOpt"></OptDlg>
	</HMContainer>
	<SqlAmisEdit v-else :supervisor="sqlEditMapSupervisor" @backLastPage="backLastPage"></SqlAmisEdit>
</template>

<script setup lang="ts">
import { deleteCurrentRow } from '/@/utils/util';
import { useDraggable } from '@vueuse/core';

import { onMounted, ref } from 'vue';
import HMContainer from '/@/components/layout/HMContainer.vue';
import { usePageDisplay } from '/@/hooks/usePageDisplay';
import { useQueryTable } from '/@/hooks/useQueryTable';
// import { useTableSort } from '/@/hooks/useTableSort';
// import { useValidateUniqueness } from '/@/hooks/useValidateUniqueness';
import Chat from '/@/components/chat/Chat.vue';
import SqlAmisEdit from './edit/SqlAmisEdit.vue';
import OptDlg from './optDlg/OptDlg.vue';
import { SupervisorPublished, supervisorPublishedMap } from './types';
import * as supervisorAdminApi from '/@/api/supervisorAdmin';
import { checkSupervisorValidate, updatePublishStatus } from '/@/api/supervisorAdmin';
import JsonEditor from '/@/components/form/jsonEditor/JsonEditor.vue';
import amisLogo from '/@/assets/logo/amis-logo.png';
import { gotoRoute } from '/@/utils/route';
import { useCompRef } from '/@/utils/types';
import { nextTick } from 'vue';
//#region ====================== 表格数据，table init ======================
const tableLoading = ref(false);
const tableData = ref([]);
const isDragStatus = ref(false);
const getTableData = async () => {
	tableLoading.value = true;
	const res = await supervisorAdminApi.getSupervisorList().finally(() => {
		tableLoading.value = false;
	});
	tableData.value = (res.values || []).map((item) => {
		item.create_time = item.create_time?.slice(0, 10);
		item.update_time = item.create_time?.slice(0, 10);

		return item;
	});
};
//#endregion

//#region ====================== 表格查询、排序，search form init ======================

const queryParams = ref({
	title: '',
});
const { resetQuery, handleQueryTable, displayTableData } = useQueryTable(tableData, queryParams, () => {
	displayTableData.value = tableData.value;
});

//#endregion

//#region ====================== 查询快捷键 ======================
const queryFormRef = ref(null);
const pressEnterSearch = (ev: KeyboardEvent) => {
	if (ev.key === 'Enter') {
		handleQueryTable();
	}
};
usePageDisplay(
	() => {
		queryFormRef.value?.$el?.addEventListener('keypress', pressEnterSearch);
	},
	() => {
		queryFormRef.value?.$el?.removeEventListener('keypress', pressEnterSearch);
	}
);
//#endregion

//#region ====================== sql ======================

const supervisorMgrShow = ref(true);
const sqlEditMapSupervisor = ref(null);
const editSqlClick = (item) => {
	sqlEditMapSupervisor.value = item;
	supervisorMgrShow.value = false;
};

const backLastPage = () => {
	supervisorMgrShow.value = true;
};
//#endregion

//#region ====================== 添加修改操作 ======================
const optDlgIsShow = ref(false);
const optDlgMapRow = ref(null);
const openOptDlg = (row?: any) => {
	optDlgMapRow.value = row;
	optDlgIsShow.value = true;
};

const updateOpt = (formValue) => {
	const foundIndex = tableData.value.findIndex((item) => item.id === formValue.id);
	if (foundIndex > -1) {
		tableData.value[foundIndex] = {
			...tableData.value[foundIndex],
			...formValue,
		};
	}
};

const insertOpt = (newData) => {
	tableData.value.unshift({ ...newData, published: SupervisorPublished.N });
};
//#endregion
//#region ====================== 改变发布状态 ======================
const publishStatusChange = async (isPublished: SupervisorPublished, id, index) => {
	const res = await updatePublishStatus({
		id: id,
		publish: isPublished,
	});
	displayTableData.value[index].published = isPublished;
};
//#endregion

//#region ====================== 跳转低代码编辑页面 ======================
const gotoAmisPage = (row) => {
	window.currentPage = row;
	gotoRoute({
		name: 'AmisEditor',
	});
};
//#endregion

//#region ====================== Chat 测试 ======================

const chatRef = useCompRef(Chat);
const chatTestMapRow = ref(null);
const chatTestIsShow = ref(false);
const openChatTest = (row) => {
	chatTestMapRow.value = row;
	chatTestIsShow.value = true;

	nextTick(() => {
		chatRef.value.clear();
		chatRef.value.autoSend(row.prompt);
	});
};

const closeChatTest = () => {
	chatTestMapRow.value = null;
	chatTestIsShow.value = false;
};

const draggableChatRef = ref<HTMLElement | null>(null);
const chatDragHandlerRef = ref<HTMLDivElement>(null);
const chatDragContainerRef = ref<HTMLDivElement>(null);

// `style` will be a helper computed for `left: ?px; top: ?px;`
const { x, y, style } = useDraggable(draggableChatRef, {
	handle: chatDragHandlerRef,
	initialValue: {
		x: document.body.clientWidth / 2 - 350,
		y: document.body.clientHeight / 2 - 400,
	},
});

const questionAi = async (text) => {
	const res = await checkSupervisorValidate(
		{
			id: chatTestMapRow.value.id,
			question: text,
		},
		{
			loading: false,
		}
	);
	return res;
};
//#endregion
onMounted(() => {
	getTableData();
});
</script>
<style scoped lang="scss"></style>
